-- ===========================================================================
--- Class TTrace
--
-- This class determines when to log combat.
-- ===========================================================================
do
  local nIsOnce = 0
  local nIsOn   = 1
  local nIsMute = 0

-- -------------------------------------------------------------------------
--- Method MUTE
--
-- This method turns off the sound.
-- -------------------------------------------------------------------------
  local function MUTE()
    nIsMute = 1
  end

-- -------------------------------------------------------------------------
--- Method UnMUTE
--
-- This method turns on the sound.
-- -------------------------------------------------------------------------
  local function UnMUTE()
    nIsMute = 0
  end

-- -------------------------------------------------------------------------
--- Method ON
--
-- This method turns on the combat log.
-- -------------------------------------------------------------------------
  local function ON()
    LoggingCombat(true) 
    TRONFrameText:SetText("TRON")
    nIsOn = 1
    DEFAULT_CHAT_FRAME:AddMessage("[TRON] says: Combat log is turned on.", 0.0, 0.85, 0.0)

    if (nil ~= Broker_TRON.obj) then
      Broker_TRON.obj.icon = "Interface\\Addons\\TRON\\images\\bit_yes"
      Broker_TRON.obj.text = "ON"
    end

    if (0 == nIsMute) then
      PlaySoundFile("Interface\\AddOns\\TRON\\sounds\\bityes.mp3")
    end
  end

-- -------------------------------------------------------------------------
--- Method OFF
--
-- This method turns off the combat log.
-- -------------------------------------------------------------------------
  local function OFF()
    LoggingCombat(false)
    TRONFrameText:SetText("TROFF")
    nIsOn = 0
    DEFAULT_CHAT_FRAME:AddMessage("[TRON] says: Combat log is turned off.", 0.0, 0.85, 0.0)

    if (nil ~= Broker_TRON.obj) then
      Broker_TRON.obj.icon = "Interface\\Addons\\TRON\\images\\bit_default"
      Broker_TRON.obj.text = "OFF"
    end

    if (0 == nIsMute) then
      PlaySoundFile("Interface\\AddOns\\TRON\\sounds\\bitno.mp3")
    end
  end
  
-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method determines whether to turn on or off the trace.
-- @param self The given TTrace object.
-- -------------------------------------------------------------------------
  local function Toggle(self)
    self.m_vecLocation.nIsInInstance, 
    self.m_vecLocation.strInstanceType = IsInInstance()

    if   (1      == self.m_vecLocation.nIsInInstance)   then
      if ("raid" == self.m_vecLocation.strInstanceType) then
        self.ON()
      else
        if (1 == nIsOn) then
          self.OFF()
        end
      end
    else
      if (1 == nIsOn) then
        TRONFrameText:SetText("TROFF")
        LoggingCombat(false)

        if   (1 == nIsOnce) then
          if (0 == nIsMute) then
            PlaySoundFile("Interface\\AddOns\\TRON\\sounds\\endline.mp3")
          end
        else
          local strName
          local strRealm

          strName, strRealm = UnitName("player")          

          if (nil ~= strRealm) then
            DEFAULT_CHAT_FRAME:AddMessage(string.format("[TRON] says: Greetings %s of %s. Got any upgrades?", strName, strRealm), 0.0, 0.85, 0.0)
          else
            DEFAULT_CHAT_FRAME:AddMessage(string.format("[TRON] says: Greetings %s. Got any upgrades?", strName, strRealm),       0.0, 0.85, 0.0)
          end

          DEFAULT_CHAT_FRAME:AddMessage(string.format("[TRON] says: Type /tr for options."), 0.0, 0.85, 0.0)

          if (0 == nIsMute) then
            PlaySoundFile("Interface\\AddOns\\TRON\\sounds\\grtprgrm.mp3")
          end

          nIsOnce = 1
        end

        nIsOn = 0
        DEFAULT_CHAT_FRAME:AddMessage("[TRON] says: Combat log is turned off.", 0.0, 0.85, 0.0)
      end
    end

  end

-- -------------------------------------------------------------------------
--- Method Update
--
-- This method Updates the zone.
-- @param self The given TTrace object.
-- -------------------------------------------------------------------------
  local function Update(self)
    self.m_vecLocation.strZone    = GetZoneText()
    self.m_vecLocation.strSubZone = GetSubZoneText()
    SetMapToCurrentZone()
  end


-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
      return "TTrace"
   end

-- -------------------------------------------------------------------------
-- Method new_TTrace
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TTrace()

      local obj =
      {
        GetType = GetType,
        MUTE    = MUTE,
        OFF     = OFF,
        ON      = ON,
        Toggle  = Toggle,
        UnMUTE  = UnMUTE,
        Update  = Update,

        m_vecLocation =
        {
          strInstanceType = "",
          strZone         = "",
          strSubZone      = "",
          nX              = 0.0,
          nY              = 0.0,
          nIsInInstance   = 1,
        },
      }

      return obj
  end 
end


-- ===========================================================================
--- Class TSemaphore
--
-- This class defines a counting semaphore.
-- ===========================================================================
do
-- -------------------------------------------------------------------------
--- Method IsLocked
--
-- This method returns true if this counting semaphore has a lock.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function IsLocked(self)
    if 1 > self.nLockCount then
      return false
    else
      return true
    end
  end

-- -------------------------------------------------------------------------
--- Method IsUnLocked
--
-- This method returns true if this counting semaphore does not have a lock.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function IsUnLocked(self)
    if 0 < self.nLockCount then
      return false
    else
      return true
    end
  end

-- -------------------------------------------------------------------------
--- Method Lock
--
-- This method locks this counting semaphore and increases the lock count.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function Lock(self)
    self.nLockCount = self.nLockCount + 1
  end

-- -------------------------------------------------------------------------
--- Method UnLock
--
-- This method unlocks this counting semaphore and decreases the lock count.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function UnLock(self)
    if (0 < self.nLockCount) then
      self.nLockCount = self.nLockCount - 1
    end
  end

-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TSemaphore"
  end

-- -------------------------------------------------------------------------
--- Method new_TSemaphore
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TSemaphore()
    local obj =
    {
      GetType    = GetType,
      IsLocked   = IsLocked,
      IsUnLocked = IsUnLocked,
      Lock       = Lock,
      UnLock     = UnLock,
      nLockCount = 0,
    }

    return obj
  end 
end


-- ===========================================================================
--- Class TTRON
--
-- This class encapsulates the WoW API.
-- ===========================================================================
do
  -- Protect event handlers against WoW patches and programmer error.

  local semEventPLAYER_ENTERING_WORLD = new_TSemaphore()
  local semEventZONE_CHANGED          = new_TSemaphore()
  local semEventRAID_ROSTER_UPDATE    = new_TSemaphore()
  local nIsDisplayed = 1

-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method shows/hides the status bar.
-- @param self The given TTRON object.
-- -------------------------------------------------------------------------
   local function IsDisplayed(self)
     return nIsDisplayed
   end

-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method shows/hides the status bar.
-- @param The given TTRON object.
-- -------------------------------------------------------------------------
  local function Toggle(self)
    if (0 == nIsDisplayed) then 
      TRONFrame:Show()
      nIsDisplayed = 1
    else
      TRONFrame:Hide()
      nIsDisplayed = 0
    end
  end

-- -------------------------------------------------------------------------
--- Method OnLoad
--
-- This method registers WoW API events.
-- @param frame The given status button window.
-- -------------------------------------------------------------------------
  local function OnLoad(frame)
    frame:RegisterEvent("PLAYER_ENTERING_WORLD")
    frame:RegisterEvent("RAID_ROSTER_UPDATE")
    frame:RegisterEvent("ZONE_CHANGED")
    frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
    frame:RegisterForClicks("RightButtonUp")
    frame:RegisterForDrag("LeftButton")
  end


-- -------------------------------------------------------------------------
--- Method HandleEventPLAYER_ENTERING_WORLD
--
-- This method handles WoW API event "PLAYER_ENTERING_WORLD".
-- -------------------------------------------------------------------------
  local function HandleEventPLAYER_ENTERING_WORLD()
    if semEventPLAYER_ENTERING_WORLD:IsUnLocked() then
      semEventPLAYER_ENTERING_WORLD:Lock()

      tblTrace:Toggle()

      semEventPLAYER_ENTERING_WORLD:UnLock()
    end
  end


-- -------------------------------------------------------------------------
--- Method HandleEventRAID_ROSTER_UPDATE
--
-- This method handles WoW API event "RAID_ROSTER_UPDATE".
-- -------------------------------------------------------------------------
  local function HandleEventZONE_CHANGED()
    if semEventZONE_CHANGED:IsUnLocked() then
      semEventZONE_CHANGED:Lock()

      tblTrace:Update()

      semEventZONE_CHANGED:UnLock()
    end
  end

-- -------------------------------------------------------------------------
--- Method HandleEventZONE_CHANGED
--
-- This method handles WoW API event "ZONE_CHANGED".
-- -------------------------------------------------------------------------
  local function HandleEventZONE_CHANGED()
    if semEventZONE_CHANGED:IsUnLocked() then
      semEventZONE_CHANGED:Lock()

      tblTrace:Update()

      semEventZONE_CHANGED:UnLock()
    end
  end

-- -------------------------------------------------------------------------
--- Method HandleEventZONE_CHANGED_NEW_AREA
--
-- This method handles WoW API event "ZONE_CHANGED_NEW_AREA".
-- -------------------------------------------------------------------------
  local function HandleEventZONE_CHANGED_NEW_AREA()
    if semEventZONE_CHANGED:IsUnLocked() then
      semEventZONE_CHANGED:Lock()

      tblTrace:Update()

      semEventZONE_CHANGED:UnLock()
    end
  end


-- -------------------------------------------------------------------------
--- Method OnEvent
--
-- This method handles WoW API events.
-- @param frame The given status button window.
-- @param event The given WoW event.
-- -------------------------------------------------------------------------
  local function OnEvent(frame, event, ...)
        if event == "PLAYER_ENTERING_WORLD" then HandleEventPLAYER_ENTERING_WORLD()
    elseif event == "ZONE_CHANGED"          then HandleEventZONE_CHANGED()
    elseif event == "ZONE_CHANGED_NEW_AREA" then HandleEventZONE_CHANGED_NEW_AREA()
    end
  end

-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TTRON"
  end

-- -------------------------------------------------------------------------
--- Method new_TTRON
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TTRON()
    local obj =
    {
        GetType = GetType,
        OnEvent = OnEvent,
        OnLoad  = OnLoad,
        Toggle  = Toggle,
        IsDisplayed = IsDisplayed,
    }

    obj.panel = CreateFrame( "Frame", "MyAddonPanel", UIParent );
    obj.panel.name = "TRON"

    local strTitle = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
    local strOverview = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine2 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine3 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine4 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine5 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine6 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strTableHeaderCol1 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    local strTableHeaderCol2 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    local strCell11 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell12 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell21 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell22 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell31 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell32 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell41 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell42 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")

    strTitle:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -15)
    strOverview:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -45)
    strLine2:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -60)
    strLine3:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -75)
    strLine4:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -90)
    strLine5:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -105)
    strLine6:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -120)
    strTableHeaderCol1:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -150)
    strTableHeaderCol2:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -150)
    strCell11:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -165)
    strCell21:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -180)
    strCell31:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -195)
    strCell41:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -210)
    strCell12:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -165)
    strCell22:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -180)
    strCell32:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -195)
    strCell42:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -210)

    strTitle:SetTextColor(0.0, 0.85, 0.0)
    strOverview:SetTextColor(1.0, 1.0, 1.0)
    strLine2:SetTextColor(1.0, 1.0, 1.0)
    strLine3:SetTextColor(1.0, 1.0, 1.0)
    strLine4:SetTextColor(1.0, 1.0, 1.0)
    strLine5:SetTextColor(1.0, 1.0, 1.0)
    strLine6:SetTextColor(1.0, 1.0, 1.0)
    strCell11:SetTextColor(1.0, 1.0, 1.0)
    strCell21:SetTextColor(1.0, 1.0, 1.0)
    strCell31:SetTextColor(1.0, 1.0, 1.0)
    strCell41:SetTextColor(1.0, 1.0, 1.0)
    strCell12:SetTextColor(1.0, 1.0, 1.0)
    strCell22:SetTextColor(1.0, 1.0, 1.0)
    strCell32:SetTextColor(1.0, 1.0, 1.0)
    strCell42:SetTextColor(1.0, 1.0, 1.0)
    strTitle:SetText("TRON V" ..GetAddOnMetadata("TRON", "Version" ) .. "   by " .. GetAddOnMetadata("TRON", "Author"))
    strOverview:SetText("Limiting the combat log to just what happens in a raid is")
    strLine2:SetText   ("essential when performing log analysis at the World of Logs")
    strLine3:SetText   ("website, www.worldoflogs.com.  TRON is an add-on whose")
    strLine4:SetText   ("primary purpose is to do just that by turning on combat")
    strLine5:SetText   ("logging when entering a raid and turning off combat logging")
    strLine6:SetText   ("when leaving a raid.")
    strTableHeaderCol1:SetText("Slash Command")
    strTableHeaderCol2:SetText("Description")
    strCell11:SetText("/tr")
    strCell12:SetText("display slash command options")
    strCell21:SetText("/tr on")
    strCell22:SetText("force combat logging on")
    strCell31:SetText("/tr off")
    strCell32:SetText("force combat logging off")
    strCell41:SetText("/tr config")
    strCell42:SetText("display configuration options")             

    obj.panel.strTitle = strTitle
    obj.panel.strOverview = strOverview
    obj.panel.strLine2 = strLine2
    obj.panel.strLine3 = strLine3
    obj.panel.strLine4 = strLine4
    obj.panel.strLine5 = strLine5
    obj.panel.strLIne6 = strLine6
    obj.panel.strTableHeader = strTableHeader
    obj.panel.strCell11 = strCell11
    obj.panel.strCell21 = strCell21
    obj.panel.strCell31 = strCell31
    obj.panel.strCell41 = strCell41
    obj.panel.strCell12 = strCell12
    obj.panel.strCell22 = strCell22
    obj.panel.strCell32 = strCell32
    obj.panel.strCell42 = strCell42    
    obj.panel:Hide()
    InterfaceOptions_AddCategory(obj.panel);
    return obj
  end 
end

tblTrace = new_TTrace()
tblTRON  = new_TTRON()

-- -------------------------------------------------------------------------
--- Function SlashCmdTronHandler
--
-- This function turns on combat logging.
-- -------------------------------------------------------------------------
local function SlashCmdTronHandler()
  if (nil ~= Broker_TRON) then
    Broker_TRON.obj.label = "TR"
  end
  tblTrace.ON()
end

-- -------------------------------------------------------------------------
--- Function SlashCmdTroffHandler
--
-- This function turns off combat logging.
-- -------------------------------------------------------------------------
local function SlashCmdTroffHandler()
  if (nil ~= Broker_TRON) then
    Broker_TRON.obj.label = "TR"
  end
  tblTrace.OFF()
end

-- -------------------------------------------------------------------------
--- Function SlashCmdTraceHandler
--
-- This function does one of the following:
--    - ''       display slash command options
--    - 'on'     force combat logging on
--    - 'off'    force combat logging off
--    - 'config' display configuration options
-- @param val The given value.
-- -------------------------------------------------------------------------
local function SlashCmdTraceHandler(val)
  if (nil ~= Broker_TRON) then
    Broker_TRON.obj.label = "TR"
  end

  if val == "on" or val == "ON" or val == "On" then
    tblTrace:ON()
  else
    if val == "off" or val == "OFF" or val == "Off" then
      tblTrace:OFF()
    else
      if val == "" or val == "?" or val == "help" or val == "HELP" or val == "Help" then
        strMessage = string.format("[TRON] says: Here are my slash commands:\n%3s\n%3s\n%3s", 
              "/tr on      - force combat logging on",
              "/tr off      - force combat logging off",
              "/tr config - display configuration options")
        DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)

        if (nil ~= Broker_TRON) then
          Broker_TRON.obj.text = ""
        end
      else
        if val == "config" or val == "CONFIG" or val == "Config" then
          if (nil ~= Broker_TRON) then
            Broker_TRON.obj.text = "CONFIG"
          end

          InterfaceOptionsFrame_OpenToCategory(tblTRON.panel);
        end
      end
    end
  end
end 

-- -------------------------------------------------------------------------
--- Function SlashCmdSoundHandler
--
-- This function does one of the following:
--    - ''       display slash command options
--    - 'on'     un-mute
--    - 'off'    mute
-- @param val The given value.
-- -------------------------------------------------------------------------
local function SlashCmdSoundHandler(val)
  if (nil ~= Broker_TRON) then
      Broker_TRON.obj.label = "SND"
  end

  if val == "on" or val == "ON" or val == "On" then
    tblTrace:UnMUTE()

    if (nil ~= Broker_TRON) then
      Broker_TRON.obj.text = "ON"
    end
  else
    if val == "off" or val == "OFF" or val == "Off" then
      tblTrace:MUTE()

      if (nil ~= Broker_TRON) then
        Broker_TRON.obj.text = "OFF"
      end
    else
      if val == "mute" or val == "MUTE" or val == "Mute" then
        tblTrace:MUTE()
        if (nil ~= Broker_TRON) then
          Broker_TRON.obj.text = "MUTE"
        end
      else 
        if val == "unmute" or val == "UNMUTE" or val == "UnMute" or val == "UnMUTE" then
          tblTrace:UnMUTE()      

          if (nil ~= Broker_TRON) then
            Broker_TRON.obj.text = "UNMUTE"
          end
        else
          if val == "" or val == "?" or val == "help" or val == "HELP" or val == "Help" then
            strMessage = string.format("[TRON] says: Here are my sound commands:\n%3s\n%3s", 
                  "/snd on      - un-mute",
                  "/snd off      - mute")
            DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)

            if (nil ~= Broker_TRON) then
              Broker_TRON.obj.text = ""
            end
          else
            if val == "config" or val == "CONFIG" or val == "Config" then
              if (nil ~= Broker_TRON) then
                Broker_TRON.obj.text = "CONFIG"
              end
              InterfaceOptionsFrame_OpenToCategory(tblTRON.panel);
            end
          end
        end
      end
    end
  end

  return
end 


-- -------------------------------------------------------------------------
--- Function SlashCmdGuiHandler
--
-- This function does one of the following:
--    - ''       display slash command options
--    - 'show'   shows the status button
--    - 'hide'   hides the status button
-- @param val The given value.
-- -------------------------------------------------------------------------
local function SlashCmdGuiHandler(val)
  if (nil ~= Broker_TRON) then
    Broker_TRON.obj.label = "GUI"
  end

  if val == "on" or val == "ON" or val == "On" then
    TRONFrame:Show()
    if (nil ~= Broker_TRON) then
      Broker_TRON.obj.text = "ON"
    end
  else
    if val == "off" or val == "OFF" or val == "Off" then
      TRONFrame:Hide()
      if (nil ~= Broker_TRON) then
        Broker_TRON.obj.text = "OFF"
      end
    else
      if val == "show" or val == "SHOW" or val == "Show" then
        TRONFrame:Show()
      if (nil ~= Broker_TRON) then
        Broker_TRON.obj.text = "SHOW"
      end
      else 
        if val == "hide" or val == "HIDE" or val == "Hide" then
          TRONFrame:Hide()      
      if (nil ~= Broker_TRON) then
        Broker_TRON.obj.text = "HIDE"
      end
        else
          if val == "" or val == "?" or val == "help" or val == "HELP" or val == "Help" then
            strMessage = string.format("[TRON] says: Here are my gui commands:\n%3s\n%3s", 
                  "/gui show  - show the status bar",
                  "/gui hide  - hide the status bar")
            DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)
      if (nil ~= Broker_TRON) then
        Broker_TRON.obj.text = ""
      end
          else
            if val == "config" or val == "CONFIG" or val == "Config" then
              InterfaceOptionsFrame_OpenToCategory(tblTRON.panel);
            end
          end
        end
      end
    end
  end

  return
end 

SLASH_TRON1,  SLASH_TRON2,  SLASH_TRON3                = "/tron",  "/TRON",  "/Tron"
SLASH_TROFF1, SLASH_TROFF2, SLASH_TROFF3               = "/troff", "/TROFF", "/Troff"
SLASH_TRACE1, SLASH_TRACE2, SLASH_TRACE3, SLASH_TRACE4 = "/tr",    "/TRACE", "/Trace", "trace"
SLASH_SOUND1, SLASH_SOUND2, SLASH_SOUND3, SLASH_SOUND4 = "/snd",   "/SOUND", "/Sound", "sound"
SLASH_GUI1, SLASH_GUI2                                 = "/gui",   "/GUI"
SlashCmdList["TRON"]  = SlashCmdTronHandler
SlashCmdList["TROFF"] = SlashCmdTroffHandler
SlashCmdList["TRACE"] = SlashCmdTraceHandler
SlashCmdList["SOUND"] = SlashCmdSoundHandler
SlashCmdList["GUI"]   = SlashCmdGuiHandler
